/* Copyright (c) 1990 by Carrick Sean Casey. */
/* For copying and distribution information, see the file COPYING. */

/* TCL interface */

#include "fn.h"
#include "externs.h"
#include <sys/dir.h>
#include <sys/file.h>

Tcl_Interp *interp;

/* function definitions */
int fset();
int c_help();
int c_hush();
int c_log();
int c_quit();
int c_rand();
int c_replay();
int c_shell();
int c_time();
int c_usage();
int s_beep();
int s_group();
int s_personal();
int s_send();
int s_user();
int s_version();
int s_who();

/* we create a lot of commands */
#define TCL_CREATE(x) Tcl_CreateCommand(interp, "x", x, (ClientData) 0);

/* initialize TCL */
tclinit()
{
	char *p;
	extern char *defprocs[];
	int x;

	/* initialize the interpreter */
	interp = Tcl_CreateInterp();

	/* add the client primitives */
	TCL_CREATE(fset)
	TCL_CREATE(c_log)
	TCL_CREATE(c_help)
	TCL_CREATE(c_hush)
	TCL_CREATE(c_quit)
	TCL_CREATE(c_rand)
	TCL_CREATE(c_replay)
	TCL_CREATE(c_shell)
	TCL_CREATE(c_time)
	TCL_CREATE(c_usage)
	TCL_CREATE(s_beep)
	TCL_CREATE(s_group)
	TCL_CREATE(s_personal)
	TCL_CREATE(s_send)
	TCL_CREATE(s_user)
	TCL_CREATE(s_version)
	TCL_CREATE(s_who)

	/* set up the default bindings */
	for (x = 0; defprocs[x] != 0; x++)
		if (Tcl_Eval(interp, defprocs[x], 0, &p) != TCL_OK) {
			printf("Default binding failed: %s\r\n", defprocs[x]);
			if (*interp->result != 0)
				puts(interp->result);
		}

}

tclreadfnrc()
{
	char *p, *path;
	char cmd[MAXNAMLEN+10];

	if ((path = getrcname()) == NULL)
		return;

	sprintf(cmd, "source %s", path);
	if (Tcl_Eval(interp, cmd, 0, &p) != TCL_OK) {
		printf("Error in rc file \"%s\".\r\n", path);
		if (*interp->result != 0)
			puts(interp->result);
	}
}


/* find the pathname of the rc file to read */
/* FNRC optionally contains the pathname of the rc file */
/* if FNRC exists, then an error will be printed if it can't be read */

char *
getrcname()
{
	char *rcfile, *homedir, *getenv();
	static char rcpath[MAXNAMLEN+1];	/* path of startup file */

	/* get name of file to use */
	if ((rcfile = getenv("FNRC")) != NULL) {

		if (*rcfile != '/') {
			/* relative to HOME */
			if ((homedir = getenv("HOME")) == NULL) {
			      printf("fn: $HOME not set - can't read \"%s\"\r\n.",
				rcfile);
			      return(NULL);
			}
			sprintf(rcpath, "%s/%s", homedir, rcfile);
			rcfile = rcpath;
		}

		/* make sure the file is there */
		if (access(rcfile, (F_OK | R_OK)) < 0) {
			printf("fn: can't read rc file \"%s\".\r\n", rcfile);
			return(NULL);
		}

	} else {
		if ((homedir = getenv("HOME")) == NULL) {
			printf("fn: warning - $HOME not set.\r\n");
			return(0);
		}
		sprintf(rcpath, "%s/.fnrc", homedir);
		rcfile = rcpath;
	}

	/* see if the file is there */
	if (access(rcfile, (F_OK | R_OK)) < 0)
		return(0);

	return(rcfile);
}

tclrestrict()
{
	/* delete commands that might shellout or read or write files */
	Tcl_DeleteCommand(interp, "exec");
	Tcl_DeleteCommand(interp, "print");
	Tcl_DeleteCommand(interp, "file");
	Tcl_DeleteCommand(interp, "source");
}
