/* Copyright (c) 1990 by Carrick Sean Casey. */
/* For copying and distribution information, see the file COPYING. */

/* This file contains routines that are unix dependent. */
/* Eventually, most unixisms should be moved here. */

#include "fn.h"
#include "externs.h"
#include <pwd.h>
#include <sgtty.h>
#include <sys/dir.h>

char *getlogin();
struct sgttyb origtty;	/* holds the user's original term settings */
int badttyinfo = 0;	/* used when running under some weird modes */

/* get a user's login id */
/* returns pointer to ID on success, 0 on failure */

char *
getloginid()
{
	char *loginid;
	struct passwd *pw;

	if (((loginid = getlogin()) == NULL) || *loginid == '\0')
		if ((pw = getpwuid(getuid())) == NULL)
			loginid = NULL;
		else
			loginid = pw->pw_name;
	return(loginid);
}


/* stash the current time in curtime */
void
gettime()
{
	time(&curtime);
}


/* set line buffering for an open file pointer */
/* output will be flushed every newline */

linebuffer(fp)
FILE *fp;
{
	setlinebuf(fp);
}


/* push a character into the terminal's input buffer */
pushback(c)
char c;
{
	if (ioctl(0, TIOCSTI, &c) < 0)
		perror("TIOCSTI ioctl failed");
}


/* get the original terminal settings */
/* stash them in origtty */

getterm()
{
	struct ltchars lt;

	/* get tty settings */
 	if (ioctl(0, TIOCGETP, &origtty) < 0) {
		badttyinfo++;
		ttyinfo.erase = '\b';	/* ^H */
		ttyinfo.kill = '\025';	/* ^U */
	} else {
		ttyinfo.erase = origtty.sg_erase;
		ttyinfo.kill = origtty.sg_kill;
	}

	/* get local special chars */
 	if (ioctl(0, TIOCGLTC, &lt) < 0) {
		ttyinfo.redraw = '\022'; /* ^R */
		ttyinfo.werase = '\027'; /* ^W */
	} else {
		ttyinfo.redraw = lt.t_rprntc;
		ttyinfo.werase = lt.t_werasc;
	}

	/* get the current window size */
	getwinsize();
}



/* set up terminal modes optimal for fn */

fnterm()
{
	struct sgttyb tty;

	if (badttyinfo)
		return;

	bcopy((char *)&origtty, (char *)&tty, (unsigned)sizeof(struct sgttyb));

	/* turn on cbreak - turn off echo */
	tty.sg_flags |= CBREAK;
	tty.sg_flags &= ~ECHO;

	/* set the new flags */
	if (ioctl(0, TIOCSETN, &tty) < 0) {
		perror("TIOCSETN (tty) failed");
		exit(-1);
	}
}


/* restore term to original settings */

restoreterm()
{
	if (badttyinfo)
		return;

	if (ioctl(0, TIOCSETN, &origtty) < 0) {
		perror("TIOCSETN (tty) failed");
		exit(-1);
	}
}

/* get the number of lines and columns for this tty */
/* stash them in origtty */

getwinsize()
{
	struct winsize win;

	/* get tty settings */
 	if (ioctl(0, TIOCGWINSZ, &win) < 0) {
		ttyinfo.rows = 24;
		ttyinfo.cols = 80;
	} else {
		ttyinfo.rows = win.ws_row;
		ttyinfo.cols = win.ws_col;
	}
}

char *
tildexpand(s)
char *s;
{
	struct passwd *pw;
	char login[17];
	char *p = login;
	static char path[MAXNAMLEN+1];

	/* has to start with a tilde */
	if (*s++ != '~')
		return(s);

	/* skim off the login id */
	while (((*s >= 'A' && *s <= 'Z') || (*s >= 'a' && *s <= 'z') ||
	  index(".-_", *s)) && ((p - login) < 17))
		  *p++ = *s++;

	/* terminate string */
	*p = '\0';

	/* get home directory from password file */
	if ((pw = getpwnam(login)) == NULL)
		return(NULL);

	strcpy(path, pw->pw_dir);
	strcat(path, s);

	return(path);
}

clearargs(argc,argv)
int argc;
char *argv[];
{
	int x;
	char *cp;

	for (x = 1; x <= argc-1; x++) {
		cp = argv[x];
		while(*cp)
			*cp++ = ' ';
	}
}
